###############################################################################
#    Copyright (c) 2017 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: documentation_test.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Sep 2017
#   Purpose: Test examples in documentation
#
#  Revision: 1
#   Comment: What's new in revision 1
#            Unlike the main unit-tests, this requires live connection.
#            Given the rate limit of 50/hr, these can't be run in a single
#            shot; although all issues are fixed, still valuable to keep
#            around. Name is purposely not following unit test standard.
#
###############################################################################
import pyunsplash
import os
import logging

api_key = os.environ.get('APPLICATION_ID', None)

# Initialize app logging
logger = logging.getLogger()
logging.basicConfig(filename='app.log', level=logging.DEBUG)

# pyunsplash logger defaults to level logging.ERROR
# If you need to change that, use getLogger/setLevel
# on the module logger, like this:
logging.getLogger("pyunsplash").setLevel(logging.DEBUG)


def funzione_1():
    pu = pyunsplash.PyUnsplash(api_key=api_key)
    return pu


def funzione_2(pu):
    logger.info('Funzione_2')
    this_user = pu.user('salvoventura', w=100, h=100)


def funzione_3(pu):
    logger.info('Funzione_3')
    # retrieve a page from the featured collections, with a maximum
    # of 5 collections per-page
    collections_page = pu.collections(type_='featured', per_page=5)


def funzione_4(pu):
    logger.info('Funzione_4')
    #
    #
    search = pu.search(type_='photos', query='red,car')
    for entry in search.entries:
        print entry.link_html


def funzione_5(pu):
    logger.info('Funzione_5')
    stats = pu.stats()
    print stats.total  # this is json


def funzione_6(pu):
    logger.info('Funzione_6')
    # use the PyUnsplash objects: all logs will be recorded to log file


# API: Class Collection

def funzione_7(pu):
    logger.info('Funzione_7')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        collection.refresh()
        print collection.id


def funzione_8(pu):
    logger.info('Funzione_8')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.id


def funzione_9(pu):
    logger.info('Funzione_9')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.title


def funzione_10(pu):
    logger.info('Funzione_10')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.description


def funzione_11(pu):
    logger.info('Funzione_11')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.user


def funzione_12(pu):
    logger.info('Funzione_12')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.link_photos


def funzione_13(pu):
    logger.info('Funzione_13')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        print collection.link_related


def funzione_14(pu):
    logger.info('Funzione_14')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        photos = collection.photos(order_by='popular', per_page=3)
        for photo in photos.entries:
            print photo.id, photo.link_download


def funzione_15(pu):
    logger.info('Funzione_15')
    collections_page = pu.collections(type_='featured', per_page=5)
    for collection in collections_page.entries:
        related_collections = collection.related
        for rel_collection in related_collections.entries:
            print rel_collection.title, rel_collection.description


# API: Class Collections

def funzione_16(pu):
    logger.info('Funzione_16')
    this_user = pu.user('salvoventura', w=100, h=100)
    collections = this_user.collections(page=1, per_page=5)
    for collection in collections.entries:
        print collection.id, collection.title


# API: Class Photo

def funzione_17(pu):
    logger.info('Funzione_17')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        photo.refresh()
        print photo.id, photo.link_download


def funzione_18(pu):
    logger.info('Funzione_18')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        photo.refresh()
        print photo.id, photo.link_download


def funzione_19(pu):
    logger.info('Funzione_19')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        print photo.id, photo.link_html


def funzione_20(pu):
    logger.info('Funzione_20')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        print photo.id, photo.link_download


def funzione_21(pu):
    logger.info('Funzione_21')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        print photo.stats


# API: Class Photos

def funzione_22(pu):
    logger.info('Funzione_22')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos()  # photos is an instance of class Photos
    for photo in photos.entries:
        print photo.id, photo.link_download


# API: Class Search

def funzione_23(pu):
    logger.info('Funzione_23')
    search = pu.search(type_='photos', query='red,car')
    for photo in search.entries:
        print photo.id, photo.link_download


# API: Class Stats

def funzione_24(pu):
    logger.info('Funzione_24')
    stats = pu.stats()
    print stats.total


# API: Class User

def funzione_25(pu):
    logger.info('Funzione_25')
    this_user = pu.user('salvoventura', w=100, h=100)
    this_user.refresh()


def funzione_26(pu):
    logger.info('Funzione_26')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.id


def funzione_27(pu):
    logger.info('Funzione_27')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.links


def funzione_28(pu):
    logger.info('Funzione_28')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.link_html


def funzione_29(pu):
    logger.info('Funzione_29')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.link_portfolio


def funzione_30(pu):
    logger.info('Funzione_30')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.link_followers


def funzione_31(pu):
    logger.info('Funzione_31')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.link_following


def funzione_32(pu):
    logger.info('Funzione_32')
    this_user = pu.user('salvoventura', w=100, h=100)
    print this_user.link_photos


def funzione_33(pu):
    logger.info('Funzione_33')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.photos(per_page=5)
    for photo in photos.entries:
        print photo.id, photo.link_download


def funzione_34(pu):
    logger.info('Funzione_34')
    this_user = pu.user('salvoventura', w=100, h=100)
    followers = this_user.followers()
    for user in followers.entries:
        print user.id, user.body.get('first_name'), user.body.get('last_name')


def funzione_35(pu):
    logger.info('Funzione_35')
    this_user = pu.user('salvoventura', w=100, h=100)
    following = this_user.following()
    for user in following.entries:
        print user.id, user.body.get('first_name'), user.body.get('last_name')


def funzione_36(pu):
    logger.info('Funzione_36')
    this_user = pu.user('salvoventura', w=100, h=100)
    photos = this_user.likes(per_page=5)
    for photo in photos.entries:
        print photo.id, photo.link_download


def funzione_37(pu):
    logger.info('Funzione_37')
    this_user = pu.user('salvoventura', w=100, h=100)
    collections = this_user.collections(page=1, per_page=5)
    for collection in collections.entries:
        print collection.id, collection.title


# API: Class Users

def funzione_38(pu):
    logger.info('Funzione_38')
    this_user = pu.user('salvoventura', w=100, h=100)
    followers = this_user.followers()  # followers is an instance of class Users
    for user in followers.entries:
        print user.id, user.body.get('first_name'), user.body.get('last_name')


def main():
    pu = funzione_1()
    # first chunk
    funzione_2(pu)
    funzione_3(pu)
    funzione_4(pu)
    funzione_5(pu)
    funzione_6(pu)
    funzione_7(pu)
    funzione_8(pu)
    funzione_9(pu)
    funzione_10(pu)
    funzione_11(pu)
    funzione_12(pu)
    funzione_13(pu)
    funzione_14(pu)
    funzione_15(pu)
    funzione_16(pu)
    funzione_17(pu)
    funzione_18(pu)

    # second chunk
    funzione_19(pu)
    funzione_20(pu)
    funzione_21(pu)
    funzione_22(pu)
    funzione_23(pu)
    funzione_24(pu)
    funzione_25(pu)
    funzione_26(pu)
    funzione_27(pu)
    funzione_28(pu)
    funzione_29(pu)
    funzione_30(pu)
    funzione_31(pu)
    funzione_32(pu)
    funzione_33(pu)
    funzione_34(pu)
    funzione_35(pu)
    funzione_36(pu)
    funzione_37(pu)
    funzione_38(pu)


if __name__ == '__main__':
    main()



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: users.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 08 Dec 2016
#   Purpose: Handles Users, and User
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
# from .liblogging import logger
from .unpage import UnsplashPage
from .unobject import UnsplashObject
from .collections import Collections
from .photos import Photos


class Users(UnsplashPage):
    def __init__(self, api_key, url, **kwargs):
        valid_options = []
        super(Users, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield User(api_key=self.api_key, source=entry)


class User(UnsplashObject):
    def __init__(self, api_key, source, **kwargs):
        valid_options = ['w', 'h']
        # simplify for source=username
        if isinstance(source, str) and not source.startswith(self._api_root):
            # assume it's a userid
            source = "{}/users/{}".format(self._api_root, source)
        super(User, self).__init__(api_key=api_key, source=source, valid_options=valid_options, **kwargs)

    @property
    def link_html(self):
        return self.links.get('html', None)

    @property
    def link_portfolio(self):
        # no sense in retrieving the link, then the body etc...
        # unless this changes in the future
        return self.body.get('portfolio_url', None)

    @property
    def link_photos(self):
        return self.links.get('photos')

    @property
    def link_followers(self):
        return self.links.get('followers')

    @property
    def link_following(self):
        return self.links.get('following')

    def photos(self, **kwargs):
        # TODO: cache the returned object
        url = self.link_photos
        return Photos(url=url, api_key=self.api_key, **kwargs)

    def followers(self, **kwargs):
        # TODO: cache the returned object
        url = self.link_followers
        return Users(url=url, api_key=self.api_key, **kwargs)

    def following(self, **kwargs):
        # TODO: cache the returned object
        url = self.link_following
        return Users(url=url, api_key=self.api_key, **kwargs)

    def likes(self, **kwargs):
        # TODO: cache the returned object
        url = '{}/likes'.format(self.url)
        return Photos(url=url, api_key=self.api_key, **kwargs)

    def collections(self, **kwargs):
        # TODO: cache the returned object
        url = '{}/collections'.format(self.url)
        return Collections(url=url, api_key=self.api_key, **kwargs)



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: test_search.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 14 Dec 2016
#   Purpose: search unit tests
#
#  Revision: 1
#   Comment: What's new in revision 1
#            use local resources
#
###############################################################################
import responses
import json
import os
from pyunsplash import PyUnsplash
from pyunsplash.src.settings import API_ROOT

api_key = os.environ.get('APPLICATION_ID', None) or 'DUMMY_APPLICATION_ID'


class TestSearch:
    # TODO: avoid code duplication
    # Need to workout how to combine responses.activate so as to avoid
    # code duplication, as the testcases are pretty much the same for all

    # TOXINIDIR comes from tox.ini
    root_path = os.environ.get('TRAVIS_BUILD_DIR', None) or os.environ.get('TOXINIDIR', None)

    store_mapping = {
        'collections': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__search_collections_query_tree.json']),
        'photos': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__search_photos_query_blue.json']),
        'users': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__search_users_query_ventura.json'])}

    @responses.activate
    def test_search_collections(self):
        type = 'collections'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())
        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        search = pu_obj.search(type, query='tree')
        for collection in search.entries:
            print(collection.id, collection.title, collection.description, collection.user, collection.link_photos, collection.link_related)

    @responses.activate
    def test_search_photos(self):
        type = 'photos'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())
        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        search = pu_obj.search(type, query='tree')
        for photo in search.entries:
            print(photo.id, photo.link_html, photo.link_download)  # , photo.stats  # TODO: include stats in unit test

    @responses.activate
    def test_search_users(self):
        type = 'users'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())
        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        search = pu_obj.search(type, query='tree')
        for user in search.entries:
            print(user.id, user.link_html, user.link_portfolio, user.link_following, user.link_followers, user.link_photos)




###############################################################################
#    Copyright (c) 2017 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: example_collections.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 14 Dec 2016
#   Purpose: Example file for collections
#
#  Revision: 2
#   Comment: What's new in revision 2
#            Add application level logging and interaction with
#            library logging example
#
#   Comment: What's new in revision 1
#
###############################################################################
import logging
import os
from pyunsplash import PyUnsplash
api_key = os.environ.get('APPLICATION_ID', None) or 'DUMMY_APPLICATION_ID'

# Initialize app logging
logger = logging.getLogger()
logging.basicConfig(filename='app.log', level=logging.DEBUG)

# pyunsplash logger defaults to level logging.ERROR
# If you need to change that, use getLogger/setLevel
# on the module logger, like this:
logging.getLogger(PyUnsplash.logger_name).setLevel(logging.DEBUG)


# instantiate PyUnsplash object
py_un = PyUnsplash(api_key=api_key)

# Get a page from the collections api
# Parameters:
#   'type' : 'generic', 'curated', 'featured'
#            default: 'generic'
#   'page' : <int>, which page to retrieve
#   'per_page': <int>, how many collections to include in each page
#
collections_page = py_un.collections(type_='generic', per_page=3)

# iterate through all the collections retrieved in the collections_page, two ways
# 1) iterating through 'body' returns just a dictionary, no additional API call
for collection in collections_page.body:
    print('Collection as dictionary', collection.get('id'), collection.get('title'))

# 2) iterating through 'entries' returns an instance of 'Collection'
for collection in collections_page.entries:
    print('Collection as object', collection.id, collection.title)

    # a Collection object allows for a richer interaction:
    # follow the 'related' collections, if any
    related_collections = collection.related
    # it's a Collections object too
    for related in related_collections.entries:
        print('Related collection', related.id, related.title)

    # go through all the 'photos' in the collection
    photos = collection.photos()
    # two ways of iterating: '.body' and '.entries' as well
    for photo in photos.entries:
        print('Photo in collection', photo.id, photo.link_download)


# You can test for navigation links and get a new 'Collections' object
# for the next, previous, first and last pages
if collections_page.has_next:
    collections_page_next = collections_page.get_next_page()

if collections_page.has_previous:
    collections_page_prev = collections_page.get_previous_page()



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: stats.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 08 Dec 2016
#   Purpose: Handle Stats
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
# from .liblogging import logger
from .unobject import UnsplashObject


class Stats(UnsplashObject):
    def __init__(self, api_key):
        url = '{}/stats/total'.format(self._api_root)
        super(Stats, self).__init__(source=url, api_key=api_key)

    @property
    def total(self):
        return self.body



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: unobject.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Dec 2016
#   Purpose: Base class for Unsplash API objects
#
#  Revision: 1
#   Comment: What's new in revision 1
#            The idea here is that, and object can be created in two ways:
#            - by its json body
#                 the body might be partial, but still the object provides
#                 shortcut/unified methods to deal with common data extraction
#                 A reload/refresh command can be issued to fetch data from the
#                 server, and the body gets updated
#            - by its url
#                 in this case an explicit API call is made to populate body
###############################################################################
from .liblogging import logger
from .rest import Rest
from .settings import API_ROOT


class UnsplashObject(object):
    _api_root = API_ROOT

    def __init__(self, api_key, source, valid_options=None, **kwargs):
        self.api_key = api_key
        self.valid_options = valid_options
        self.body, self.url, self._agent = None, None, None

        self.query_parameters = self._sanitized_query_parameters(kwargs)
        self._parse_source(source)  # sets self.body, self.url

        if not self._agent:
            self._agent = Rest(api_key=self.api_key)

    def refresh(self):
        if not self._agent:
            logger.debug('need an agent first')
            self._agent = Rest(api_key=self.api_key)

        logger.debug('object refresh from url %s', self.url)
        r = self._agent.get(self.url, self.query_parameters)
        if r.status_code == 200:
            logger.debug('status %s: loading object body', r.status_code)
            self.body = r.json()
        else:
            logger.debug('status %s: object body not refreshed', r.status_code)

    @property
    def id(self):
        return self.body.get('id', None)

    @property
    def links(self):
        return self.body.get('links', None)

    def _parse_source(self, source):
        # guess format based on source type, extract the link to self
        if isinstance(source, dict):
            logger.debug('Source is a dictionary')
            self.body = source
            self.url = source.get('links').get('self')
        #                                               # TODO: maybe protect and raise appropriate exception in
        elif isinstance(source, str):                   # case someone feeds a random dictionary here
            logger.debug('Source is a string')
            if source.startswith(self._api_root):
                self.url = source
                self.refresh()
            else:
                logger.info('Source is a string, but we did not handle it: %s', source)
                raise ValueError('Source is a string, but we did not handle it: %s' % source)
        else:
            logger.info('Invalid parameter to constructor: %s', source)
            raise ValueError('Invalid parameter to constructor: %s' % source)

    def _sanitized_query_parameters(self, kwargs):
        logger.debug('call _sanitized_query_parameters(%s)', kwargs)
        query_params = {}
        for key in kwargs:
            if self.valid_options and key not in self.valid_options:
                logger.info('invalid parameter %s, safely ignoring it', key)
                continue
            query_params[key] = kwargs[key]
        logger.debug('     returning %s', query_params)
        return query_params
